#ifndef _ENTRY_CPP
#define _ENTRY_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "../../../CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void OnError(CSocketServer *pSock, CSocketClient *pClient, int iErrorNumber, const char *sErrorMsg)
{
	if(pClient)
	{
		printf("Error: %d, Client: %d, Msg: %s\n", iErrorNumber, pClient->PeerID(), sErrorMsg);
	}
	else{
		printf("Error: %d, Msg: %s\n", iErrorNumber, sErrorMsg);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ClientProc(CSocketServer *pSock, CSocketClient *pClient, LPBASICHUNK pChunk)
{
	printf("%d Connected.\n", pClient->PeerID());

	int iPackets = 0;
    while(pClient->IsConnected())
    {
		if(pClient->GetNextClientData(pChunk))
		{
			printf("Received[%d]: %d\n>", iPackets++, pChunk->iLength);
		}
	}

	printf("%d Disconnected.\n", pClient->PeerID());
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
	int iPort = 8485;
	int iMaxClients = 0;
	CSocketServer Sock;
	
	if(CreateMutex(NULL, TRUE, "CSocketServer_SelfTest"))
	{
		if(GetLastError() == ERROR_ALREADY_EXISTS)
		{
			printf("Initializing the Client.\n");
			if(!Sock.Initialize(&ClientProc, &OnError))
			{
				return 0;
			}
		}
		else{
			char sBuf[64];
			printf("How many clients would you like to test with[1 - %d]: ", FD_SETSIZE);

			gets_s(sBuf, sizeof(sBuf));
			iMaxClients = atol(sBuf);
			if(iMaxClients >= 1 && iMaxClients <= FD_SETSIZE)
			{
				printf("Initializing the Server.\n");
				if(!Sock.Initialize(iPort, iMaxClients, &OnError))
				{
					return 0;
				}
			}
			else{
				printf("Invalid number of clients.\n");
				return 0;
			}
		}
	}

	if(Sock.Start())
	{
		if(Sock.IsListeningServer())
		{
			//Launch a "bunch" of client applications.
			for(int iInstance = 0; iInstance < iMaxClients; iInstance++)
			{
				char sModule[MAX_PATH];
				GetModuleFileName(NULL, sModule, sizeof(sModule));
				ShellExecute(NULL, "OPEN", sModule, "", NULL, SW_MINIMIZE);
				Sleep(100);
			}

			//Status and speed calculation functionality.
			double dBytesSent = 0;
			int iStatCount = 0;
			clock_t ctBegin = 0;
						
			BASICHUNK Chunk;
			Sock.Alloc(&Chunk, (1024*1024));
			Chunk.iLength = Chunk.iAlloc;

			for(int iClient = 0; Sock.GetConnectedClient(&iClient);)
			{
				Sock.Client(iClient)->SetNextSendData(&Chunk);
				dBytesSent += Chunk.iLength;

				if(iStatCount++ == 10)
				{
					if(ctBegin != 0)
					{
						clock_t ctEnd = clock();
						double dElapsed = ((double) (ctEnd - ctBegin)) / CLOCKS_PER_SEC;
						double dBPS = (dBytesSent/1024.0/1024.0) / dElapsed;
						printf("  %d connections @ %.2f MB/s\n", Sock.CurrentConnections(), dBPS);
					}
					iStatCount = 0;
					dBytesSent = 0;
					ctBegin = clock();
				}
			}

			Sock.Free(&Chunk);
		}
		else{
			printf("Connecting to the server.\n");
			
			if(!Sock.ConnectUseCurrentThread("localhost", iPort))
			{
				printf("Failed to connect to the remote server.\n");
			}
		}
	}
	else{
		printf("Failed to start the service.\n");
	}

	Sock.Destroy();
	
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
