///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_ClassProperties_Cpp
#define _CSocketServer_ClassProperties_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function returns a pointer to one of the client slots.
*/
CSocketClient *CSocketServer::Client(int iSlot)
{
	if(iSlot < 0 || iSlot >= this->p_iMaxConnections)
	{
		return NULL;
	}
	return &this->p_Clients[iSlot];
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	This function can be used to poll for connected clients.
	Pass in a pointer to an integer, and it will pass out the client
		slot number of a qualified (connected and not pending disconnection) connected client.
*/
bool CSocketServer::GetConnectedClient(int *iSlot)
{
	static int iLastClient = 0;
	int iStartingPoint = iLastClient;

	if(iLastClient < 0 || iLastClient >= this->p_iMaxConnections)
	{
		iLastClient = 0;
	}

	while(this->IsActive())
	{
		//Loop from where we left off last time, to the MAX count of clients.
		for(;iLastClient < this->p_iMaxConnections; iLastClient++)
		{
			if(this->p_Clients[iLastClient].IsConnected())
			{
				*iSlot = iLastClient++;
				return true;
			}
		}
		iLastClient = 0;

		//Loop from slot 0 to where the top loop started, this makes
		//	sure we will never Sleep() while clients are connected.
		for(;iLastClient < iStartingPoint; iLastClient++)
		{
			if(this->p_Clients[iLastClient].IsConnected())
			{
				*iSlot = iLastClient++;
				return true;
			}
		}
		iStartingPoint = 0;

		Sleep(1);
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the socket server is in "packet mode", else returns false.

	Enabling / Disabling "Packet Mode" only sets the default for new connections,
		it does not change the "Packet Mode" for existing connectios.

*/
bool CSocketServer::PacketMode(void)
{
	return this->p_bPacketMode;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Enables / Disables "packet mode" for the socket server.

	Enabling / Disabling "Packet Mode" only sets the default for new connections,
		it does not change the "Packet Mode" for existing connectios.

*/
void CSocketServer::PacketMode(bool bUsePacketing)
{
	this->p_bPacketMode = bUsePacketing;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function sets the backet buffer size.
*/
void CSocketServer::PacketBufferSize(int iSize)
{
	this->p_iPacketBufferSize = iSize;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function returns the backet buffer size.
*/
int CSocketServer::PacketBufferSize(void)
{
	return this->p_iPacketBufferSize;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function returns listen port for the server.
*/
int CSocketServer::GetListenPort(void)
{
	return this->p_iListenPort;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function returns true if the client / server is running.
*/
bool CSocketServer::IsActive(void)
{
	return this->p_bActive;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to set enable the Accept / Connect functionality.
*/
void CSocketServer::AllowNewConnections(bool bAllow)
{
	this->p_bAllowNewConnections = bAllow;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function returns true if the class was initialized as a server. False is client only.
*/
bool CSocketServer::IsListeningServer(void)
{
	return this->p_bIsListeningServer;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the pointer to used defined per class instance data.
*/
VOID *CSocketServer::UserData(void)
{
	return this->pUserData;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the pointer to used defined per class instance data.
*/
void CSocketServer::UserData(VOID *pData)
{
	this->pUserData = pData;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to set p_iPacketHeader to a new value. 
*/
void CSocketServer::PacketHeader(int iPacketHeader)
{
	this->p_iPacketHeader = iPacketHeader;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to get the value of p_iPacketHeader. 
*/
int CSocketServer::PacketHeader(void)
{
	return this->p_iPacketHeader;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to set p_iNextPeerID to a new value. 
*/
void CSocketServer::NextPeerID(int iPeerID)
{
	this->p_iNextPeerID = iPeerID;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to get the value of p_iNextPeerID. 
*/
int CSocketServer::NextPeerID(void)
{
	return this->p_iNextPeerID;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to get the value of p_iMaxConnections. 
*/
int CSocketServer::MaxConnections(void)
{
	return this->p_iMaxConnections;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to get the count of currently connected clients.
*/
int CSocketServer::CurrentConnections(void)
{
	return this->p_iCurrentClients;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the current size of the internal send buffer.
*/
int CSocketServer::GetSendBufSz(void)
{
	return this->p_SendChunk.iAlloc;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the current size of the internal receive buffer.
*/
int CSocketServer::GetRecvBufSz(void)
{
	return this->p_RecvChunk.iAlloc;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the current Max Inactivity Time brfore a peer is disconnected.
*/
double CSocketServer::MaxInactivityTime(void)
{
	return this->p_dMaxInactivityTime;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Sets the Max Inactivity Time brfore a peer is disconnected.
*/
void CSocketServer::MaxInactivityTime(double dSeconds)
{
	this->p_dMaxInactivityTime = dSeconds;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
